/**
 * Type for an equality checking function.
 * It should return true if the two arguments are equal, and false otherwise.
 */
type IEquals<T> = (x: T, y: T) => boolean;

/**
 * Find a least lexicographical match for the longest common subsequence.
 *
 * The returned value is an tuple array (called `pairs`), where each element (pair=[i,j]) indicates that
 * the i-th element of the first sequence is paired with the j-th element of the second sequence.
 *
 * @param N1
 * @param N2
 * @param equals
 * @returns
 * @see https://me.guanghechen.com/post/algorithm/lcs/
 */
declare function findMinLexicographicalLCS(N1: number, N2: number, equals: IEquals<number>): Array<[number, number]>;
/**
 * Find the length of the longest common subsequence.
 *
 * @param N1
 * @param N2
 * @param equals
 * @returns
 * @see https://me.guanghechen.com/post/algorithm/lcs/
 */
declare function findLengthOfLCS(N1: number, N2: number, equals: IEquals<number>): number;
/**
 * Find Longest Common Subsequence with Dynamic Programming.
 *
 * Let dp(i,j) denote the longest common subsequence of {a1,a2,...,ai} and {b1,b2,...,bj}, it is not
 * difficult to derive the transition equation:
 *
 *    dp(i,j) = max{dp(i-1,j), dp(i,j-1), dp(i-1,j-1)+f(i,j)}, where f(i,j) = 1 if ai=aj else 0
 *
 * ### Optimization
 *
 * let dp(j) denote the longest common subsequence of {a1, a2, ...aN1} and {b1, b2, ...,bj}, then we
 * should update in reverse order, similar to the rolling array of the knapsack problem.
 *
 * The time complexity is still O(N1xN2), but the space complexity is optimized to O(N2) now.
 *
 * @param N1
 * @param N2
 * @param equals
 * @returns
 * @see https://me.guanghechen.com/post/algorithm/lcs/
 */
declare function findLCSOfEveryRightPrefix(N1: number, N2: number, equals: IEquals<number>): number[] | null;

export { findLCSOfEveryRightPrefix, findLengthOfLCS, findMinLexicographicalLCS };
